@@ -2,7 +2,7 @@
 package MRTG_lib;
 
 ###################################################################
-# MRTG 2.9.17  Support library MRTG_lib.pm
+# MRTG 2.9.19  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <oetiker@ee.ethz.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -69,7 +69,7 @@ $VERSION = 2.090017;
        [sub{int($_[0]) >= 300}, sub{"$_[0] should be 300 seconds or more"}],
 
        'interval' => 
-       [sub{int($_[0]) >= 1}, sub{"$_[0] should be at least 1 Minute"}], 
+       [sub{int($_[0]) >= 1 && int($_[0]) <= 60}, sub{"$_[0] should be at least 1 Minute and no more than 60 Minutes"}], 
 
        'writeexpires' =>  
        [sub{1}, sub{"Internal Error"}],
@@ -154,6 +154,9 @@ $VERSION = 2.090017;
        'addhead[]' => 
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
+       'rrdrowcount[]' => 
+       [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
+
        'extension[]' =>
        [sub{1}, sub{"Internal Error"}], #what ever the user chooses.
 
@@ -317,17 +320,17 @@ sub readcfg ($$$$;$$) {
     my $nextfile = $cfgfile;
     my %routerhash;
     while (1) {        
-        if (eof $hand) {
+        if (eof $hand || not defined ($_ = <$hand>) ) {
                 close $hand;
                 if (scalar @handstack){
                         $hand = pop @handstack;
                         $nextfile = pop @filestack;
+                        next;
                 } else {
                         last;
                 }
         }
         $file=$nextfile;
-        $_ = <$hand>;
         chomp;
         my $line = $.;
         if (/^include:\s*(.*?\S)\s*$/i){
@@ -525,12 +528,14 @@ sub cfgcheck ($$$$) {
     my ($routers, $cfg, $rcfg, $target) = @_;
     my ($rou, $confname, $one_option);
     my $error="no";
-    my(@known_options) = qw(growright bits noinfo absolute gauge nopercent
+    my(@known_options) = qw(growright bits noinfo absolute gauge nopercent avgpeak
 			    integer perhour perminute transparent dorelpercent 
 			    unknaszero withzeroes noborder noarrow noi noo
 			    nobanner nolegend);
 
     if (defined $$cfg{workdir}) {
+        die ("ERROR: WorkDir must not contain spaces when running on Windows. (Yeat another reason to get Linux)\n")
+                if $OS eq 'NT' and $$cfg{workdir} =~ /\s/;
         ensureSL(\$$cfg{workdir});
         $$cfg{logdir}=$$cfg{htmldir}=$$cfg{imagedir}=$$cfg{workdir};
         if (not -d $$cfg{workdir}){
@@ -639,7 +644,7 @@ sub cfgcheck ($$$$) {
     unless ($$cfg{logformat} eq 'rrdtool') {
         # interval has to be 5 minutes at least without userrdtool
         if ($$cfg{interval} < 5) {
-            die "ERROR: CFG Error in \"Interval\": should be at least 5 Minutes";
+            die "ERROR: CFG Error in \"Interval\": should be at least 5 Minutes (unless you use rrdtool)";
         }
     }
 
@@ -669,11 +674,7 @@ sub cfgcheck ($$$$) {
                 $$rcfg{'directory_web'}{$rou}="";
         }
 
-     	if (! defined $$rcfg{"pagetop"}{$rou}) {
-            warn ("WARNING: \"PageTop[$rou]\" is not specified.\n");
-            $error = "yes";
-        } else {
-            # allow for linebreaks
+     	if (defined $$rcfg{"pagetop"}{$rou}) {
             $$rcfg{"pagetop"}{$rou} =~ s/\\n/\n/g;
         }
 
@@ -1422,8 +1423,12 @@ RRD
     $rrd .= <<RRD;
 </rrd>
 RRD
-    open R, "|$$cfg{rrdtool} restore - $$cfg{logdir}$$rcfg{'directory'}{$router}$router.rrd"
-        or die "Starting $$cfg{rrdtool} restore: $!";
+        
+    if ( $OS eq 'NT' ) {
+       open (R, "|$$cfg{rrdtool} restore - $$cfg{logdir}$$rcfg{'directory'}{$router}$router.rrd");
+    } else {
+       open (R, "|-") or exec "$$cfg{rrdtool}","restore","-","$$cfg{logdir}$$rcfg{'directory'}{$router}$router.rrd";
+    }
     print R $rrd;
     close R;
 }
@@ -1509,7 +1514,8 @@ sub setup_loghandlers ($){
 		$Msg =~ s/\n/\r\n/g;
                 $Msg =~ s/[\n\r]$//g;
 		$EventLog->Report({
-		    EventID => 1000,
+ 		      EventID => 1000,
+                      Category => "WARN",
 		      EventType => $Type,
 		      Strings => $Msg });
 		$EventLog->Close;
@@ -1520,7 +1526,8 @@ sub setup_loghandlers ($){
 		$Msg =~ s/\n/\r\n/g;
                 $Msg =~ s/[\n\r]$//g;
 		$EventLog->Report({
-		    EventID => 1000,
+		      EventID => 1000,
+                      Category => "ERROR",
 		      EventType => &Win32::EventLog::EVENTLOG_ERROR_TYPE,
 		      Strings => $Msg });
 		$EventLog->Close;
@@ -1552,7 +1559,11 @@ sub setup_loghandlers ($){
 	
     }
 }    
-       
+
+__END__
+
+=pod
+
 =head1 NAME
 
 MRTG_lib.pm - Library for MRTG and support scripts
@@ -1763,14 +1774,4 @@ and other contributors, mentioned in the file C<CHANGES>
 
 Documentation by Rainer Bawidamann E<lt>Rainer.Bawidamann@rz.uni-ulm.deE<gt>
 
-=head1 LICENSE
-
-GNU General Public License
-
-=head1 COPYRIGHT
-
-MRTG_lib is  Copyright 2000 by Tobias Oetiker E<lt>tobi@oetiker.chE<gt>
-and Dave Rand E<lt>dlr@bungi.comE<gt>.
-
 =cut
-        
