@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.14  -- Rateup
+ MRTG 2.9.19  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -53,7 +53,7 @@
 /* VC++ 6.0 under Win95/NT */
 #if defined(__WATCOMC__) || defined(WIN32)
 #include <string.h>
-#include <sys\types.h>
+#include <sys/types.h>
 #include <direct.h>
 #include <io.h>
 #endif
@@ -61,7 +61,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.14";
+char *VERSION = "2.9.19";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -135,10 +135,8 @@ struct LAST {
     char in[MAXL],out[MAXL];
 } last;
 
-#if !defined(__WATCOMC__) && !defined(WIN32)
 #define max(a,b) ((a) > (b) ? (a) : (b))
 #define min(a,b) ((a) < (b) ? (a) : (b))
-#endif
 
 /*
 
@@ -205,14 +203,14 @@ for debuging
   }
 
  
-void image(file,maxvi,maxvo,maxx,maxy,xscale,yscale,growright,step, bits, ytics, yticsf, peak)
+static void image(file,maxvi,maxvo,maxx,maxy,xscale,yscale,growright,step, bits, ytics, yticsf, peak)
 char *file;
 long maxvi,maxvo;
 long maxx;
 unsigned long maxy,growright,step,bits;
 double xscale,yscale;
-int ytics;
-double yticsf;
+int ytics; /* number of tics on the y axis*/
+double yticsf; /* scale everything on the y axis with this factor*/
 int peak;
 {
     FILE *fo;
@@ -223,13 +221,13 @@ int peak;
     double inr, outr,muli, interval;
     time_t now,onow,nextnow;
     struct tm tm2, *tm=&tm2;
-    char *graph_label[600];
+    char **graph_label;
     char ylab[30];
     /* scaling helpers */
     long maxv_q;
     unsigned long valsamp,maxin,maxout,digits,maxpercent=0;
     unsigned long  sca_ten,sca_hun,nmax_q;
-    double avin, avout,latestout=0,latestin=0,nmax,avpercent=0,latestpercent=0;
+    double avmxin,avmxout, avin, avout,latestout=0,latestin=0,nmax,avpercent=0,latestpercent=0;
     double nex_ten,nex_hun,nex_rnd;
     double sca_max_q,dummy;
     double percent;
@@ -255,7 +253,12 @@ int peak;
     int i_outp, i_outpg;
     /* Dotted style */
     int styleDotted[3];
-
+    if ((graph_label=(char **)calloc(1,sizeof(char*)*maxx) ) == NULL) 
+    {
+         fprintf(stderr,"Rateup ERROR: Out of memory in graph creation\n");
+         exit(1);
+    }       
+                                        
     /* multiplicator for bits/bytes */
     muli = 7*bits+1;
     maxvi *= muli;
@@ -282,7 +285,7 @@ int peak;
 	exit(1);
     }
     onow = now;
-    avin = avout = 0.0;
+    avin = avout = avmxin = avmxout = 0.0;
     inmax = outmax = maxin = maxout = 0;
     valsamp =0;
     for (maxs = 0,n=0,x=0; x<maxx; x++) {
@@ -290,10 +293,13 @@ int peak;
 	/*scale with muli */
 	inr *=muli; outr *=muli; inmax *=muli; outmax *=muli;	
 	/* ignore times when we have not sampled */
-	if(inr>0 || outr>0 || (options & OPTION_WITHZEROES)) valsamp++;
+	if( inmax >0 || outmax > 0 || inr>0 || outr>0 || (options & OPTION_WITHZEROES)) valsamp++;
 	if (x==0) {latestin=inr;latestout=outr;if (outr) {latestpercent=inr*(double)100./outr;}}
 	avin += inr;
 	avout += outr;
+	avmxin += inmax;
+	avmxout += outmax;
+	
 	if (peak) {
 		maxin = max(maxin,inmax);
     	        maxout = max(maxout,outmax);
@@ -322,6 +328,8 @@ int peak;
     if (valsamp) {
        	avin /= valsamp;
 	avout /= valsamp;
+	avmxin /= valsamp;
+	avmxout /= valsamp;
     }
 	
     printf("%lu\n",(unsigned long)(maxin/muli));
@@ -339,8 +347,10 @@ int peak;
     if (options & OPTION_DORELPERCENT) {
        printf("%lu\n",(unsigned long)(latestpercent));
     }
-
-    if (maxv<0 || maxv < maxs) {maxv = maxs;}
+    printf("%lu\n",(unsigned long)(avmxin/muli));
+    printf("%lu\n",(unsigned long)(avmxout/muli));
+    
+    if (maxv < 0 || (unsigned long) maxv < maxs) {maxv = maxs;}
       
     now = onow;
 
@@ -367,13 +377,16 @@ int peak;
     digits = 0;
     {
     double number = (double)maxv_q;
-    number *= yticsf;
+    number *= yticsf;  /* we just want to scale the lables nothing else */ 
 
 /*
        while (number/(double) kilo >= (double)kilo && digits<kMGnumber*3) {
 */
-       while (number >= (double)kilo && digits<kMGnumber*3) {
-          number /= (double) kilo;
+       /* yes this should be kilo, but then the log and pow bits below
+          should be base 'kilo' as well and not base 10 */
+           
+       while (number >= (double)1000 && digits<(unsigned long) kMGnumber*3) {
+          number /= (double) 1000;
           digits += 3;
        }
        sca_max_q = (double) ( (int) (  ( (double)100.*(double)number )/
@@ -394,31 +407,37 @@ int peak;
           *((char*)strchr(short_si_out, ',')) = '\0';
           }
     } else {
-        short_si_out = short_si[min(digits/3,kMGnumber)];
+        short_si_out = short_si[min((signed) (digits/3), kMGnumber)];
      }
       
     if (maxv_q) {
-    	digits = log10((double)maxv_q);
+/*    	digits = log10((double)maxv_q); */
+    	digits = log10((double)maxv_q*yticsf);
     } else {
 	digits=0;
     }
 
-    sca_ten = maxv_q / pow(10.0,(double)digits);
-    sca_hun = maxv_q / pow(10.0,(double)digits-1);
+/*  sca_ten = maxv_q / pow(10.0,(double)digits); */
+/*  sca_hun = maxv_q / pow(10.0,(double)digits-1); */
+    sca_ten = (double)maxv_q*yticsf / pow(10.0,(double)digits);
+    sca_hun = (double)maxv_q*yticsf / pow(10.0,(double)digits-1);
     nex_rnd = (sca_hun) * pow(10.0,(double)digits-1);
     nex_ten = (sca_ten+1) * pow(10.0,(double)digits);
     nex_hun = (sca_hun+1) * pow(10.0,(double)digits-1);
 
-    if (nex_ten <= (1.1 * maxv_q))  {
+/*  if (nex_ten <= (1.1 * maxv_q))  { */
+    if (nex_ten <= (1.1 * maxv_q * yticsf))  {
 	nmax_q = nex_ten; 
-    } else if (maxv_q == nex_rnd) {
+/*  } else if (maxv_q == nex_rnd) {  */
+    } else if ((maxv_q * yticsf) == nex_rnd) {
 	nmax_q = nex_rnd; 
     } else {
 	nmax_q = nex_hun;
     }
 
     sca_max_q = nmax_q/(pow(10.0,(double)((int)(digits/3))*3));
-    nmax=sca_max_q*ytics*(pow((double)kilo,(double)((int)(digits/3))));
+/*  nmax=sca_max_q*ytics*(pow((double)kilo,(double)((int)(digits/3))));  */
+    nmax=(sca_max_q/yticsf)*ytics*(pow((double)kilo,(double)((int)(digits/3))));
 
     if (nmax < 1.) {nmax=1.;}
 
@@ -512,20 +531,20 @@ int peak;
 	inr *=muli; outr *=muli; inmax *=muli; outmax *=muli;		
 
 	y = ((double)inr/nmax) * maxy;
-	if (y >= maxy) y = maxy;
+	if ((unsigned long) y >= maxy) y = maxy;
 	lhist[x].in = y;
 	y = ((double)outr/nmax) * maxy;
-	if (y >= maxy) y = maxy;
+	if ((unsigned long) y >= maxy) y = maxy;
 	lhist[x].out = y;
 	y = ((double)inmax/nmax) * maxy;
-	if (y >= maxy) y = maxy;
-	if (x && y < (lhist[x-1].inmax)) {
+	if ((unsigned long) y >= maxy) y = maxy;
+	if (x && (unsigned long) y < (lhist[x-1].inmax)) {
 	  y += (lhist[x-1].inmax); y /= 2;
 	}
         lhist[x].inmax = y;
 	y = ((double)outmax/nmax) * maxy;
-	if (y >= maxy) y = maxy;
-	if (x && y < (lhist[x-1].outmax)) {
+	if ((unsigned long) y >= maxy) y = maxy;
+	if (x && (unsigned long) y < (lhist[x-1].outmax)) {
 	  y += (lhist[x-1].outmax); y /= 2;
 	}
         lhist[x].outmax = y;
@@ -645,7 +664,7 @@ int peak;
 	if (!bits) { longup = "Bytes per Second"; shortup = "Bytes/s"; }
 	else { longup = "Bits per Second"; shortup = "Bits/s"; }
     }
-    if (maxy < gdFontSmall->w*16) {	
+    if (maxy < (unsigned long) gdFontSmall->w*16) {	
 	gdImageStringUp(graph, gdFontSmall,8, ytr((maxy-gdFontSmall->w*strlen(shortup))/2),
 		      (unsigned char*)shortup, i_grid);
     } else {
@@ -665,9 +684,11 @@ int peak;
 	
 /*
     	sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si[digits/3]);
-    	sprintf(ylab,"%6.1f %s",sca_max_q*i*yticsf,short_si_out);
 */
-        sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si_out);
+/*    	sprintf(ylab,"%6.1f %s",sca_max_q*i*yticsf,short_si_out);  */
+    	sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si_out);
+
+/*        sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si_out); */
         gdImageString(graph, gdFontSmall,23, 
         	      ytr(i*maxy/ytics+gdFontSmall->h/2),
 		      (unsigned char*)ylab, i_grid);
@@ -742,10 +763,11 @@ int peak;
     gdImageDestroy(brush_outm);
     gdImageDestroy(brush_outp);
     free(lhist);
+    free(graph_label);
 }
 
 	    
-double diff(a,b)
+static double diff(a,b)
 char *a,*b;
 {
     char res[MAXL], *a1, *b1, *r1;
@@ -756,9 +778,9 @@ char *a,*b;
        a = b1;
     }
     
-    while (!isdigit(*a))
+    while (!isdigit((int)*a))
 	a++;
-    while (!isdigit(*b))
+    while (!isdigit((int)*b))
 	b++;
     a1 = &a[strlen(a)-1];
     m = max(strlen(a),strlen(b));
@@ -785,7 +807,7 @@ char *a,*b;
     }
     if (c) {
 	r1 = &res[m+1];
-	for (x=0; isdigit(*r1) && x<m; x++,r1--)  {
+	for (x=0; isdigit((int)*r1) && x<m; x++,r1--)  {
 	    *r1 = ('9' - *r1 + c) + '0';
 	    if (*r1 > '9') {
 		*r1 -= 10; 
@@ -799,7 +821,7 @@ char *a,*b;
 	return(atof(res));
 }
 
-int readhist(file)
+static int readhist(file)
 char *file;
 {
     FILE *fi;
@@ -810,7 +832,7 @@ char *file;
     time_t cur;
 
     if ((fi = fopen(file,"r")) != NULL) {
-	if (fscanf(fi,"%ld %s %s\n",&last.time,&last.in[0],&last.out[0]) != 3){
+	if (fscanf(fi,"%ld %s %s\n",(long int *)&last.time,&last.in[0],&last.out[0]) != 3){
            fprintf(stderr,"Read Error: File %s lin 1\n",file);
 	   retcode = 1;
 	}
@@ -823,7 +845,8 @@ char *file;
                  &rd[0],&rd[1],&rd[2],&rd[3],&rd[4]) <5 ) {
 	      rd[3]=rd[1];rd[4]=rd[2]; }
 	    for (i=0;i<=4;i++){
-	      rd[i]=rd[i] < 0 ? 0 : rd[i];
+	      if (rd[i] & 0x80000000)
+		rd[i] = 0;
 	    }
 					
 	    hist->time=rd[0];
@@ -864,7 +887,7 @@ char *file;
     return(retcode);
 }
 
-void readfile() {
+static void readfile() {
     char buf[128];
     int err, x;
     time_t now;
@@ -917,7 +940,7 @@ void readfile() {
     }
 }
 	     
-void update(in, out, abs_max, absupdate)
+static void update(in, out, abs_max, absupdate)
 char *in, *out;
 unsigned long abs_max;
 int absupdate;
@@ -935,7 +958,7 @@ int absupdate;
     now = NOW;
  
     if (now < last.time) {
-       fprintf(stderr,"Rateup ERROR: %s found that %s's log file time of %lu was greater than now (%lu)\nERROR: Let's not do the time warp, again. Logfile unchanged.\n",program,router,last.time,now);
+       fprintf(stderr,"Rateup ERROR: %s found that %s's log file time of %lu was greater than now (%lu)\nERROR: Let's not do the time warp, again. Logfile unchanged.\n",program,router,(unsigned long)last.time,(unsigned long)now);
        return;
     }
     sprintf(buf,"%s.tmp",router);
@@ -948,7 +971,7 @@ int absupdate;
     hist = lhist;
 
     period = ( now - last.time);
-    if (period <= 0 || period > (60*60) || last.in[0] == 'x') {	/* if last update is strange */
+    if (period <= 0 || period > (60*60) || last.in[0] == 'x' ) {	/* if last update is strange */
       if (options & OPTION_UNKNASZERO) {
 	inrate = 0;                             /* sync unknown to zero */
 	outrate = 0;
@@ -977,14 +1000,14 @@ int absupdate;
 	    outrate = diff(out,last.out) / period;
 	}
     }
-    if (inrate < 0 || inrate > abs_max) {
+    if (inrate < 0 || (unsigned) inrate > abs_max) {
       if (options & OPTION_UNKNASZERO) {
 	inrate = 0;                             /* sync unknown to zero */
       } else {
 	inrate = history[0].in;			/* Sync by using last value */
       }
     }
-    if (outrate < 0 || outrate > abs_max) {
+    if (outrate < 0 || (unsigned) outrate > abs_max) {
       if (options & OPTION_UNKNASZERO) {
 	outrate = 0;                             /* sync unknown to zero */
       } else {
@@ -992,7 +1015,7 @@ int absupdate;
       }
     }
     if ((fo = fopen(buf,"w")) != NULL) {
-	fprintf(fo,"%lu %s %s\n",now,in,out);
+	fprintf(fo,"%lu %s %s\n",(unsigned long)now,in,out);
 	last.time = now;
         /* what is this good for? */
 	/* gauge und absolute */
@@ -1003,7 +1026,7 @@ int absupdate;
 	    strcpy(last.in,in);
 	    strcpy(last.out,out);
         }
-	fprintf(fo,"%lu %lu %lu %lu %lu\n",now, inrate, outrate, inrate, outrate);
+	fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now, inrate, outrate, inrate, outrate);
 	nout = 1;
 	hist->time = now;
 	hist->in = inrate; hist->out = outrate;
@@ -1020,9 +1043,9 @@ int absupdate;
 
 	/* gobble up every shred of data we can get ... */
 	if (plannow < now) {
-	  NEXT((now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
-		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
+	  NEXT((unsigned long) (now-plannow));
+	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
+		  (unsigned long)inr,(unsigned long)outr,(unsigned long)inmax,(unsigned long)outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
 	  hist->inmax = inmax; hist->outmax = outmax;
@@ -1033,8 +1056,8 @@ int absupdate;
 
 	for (x=1; x<DAY_COUNT; x++) {
 	    NEXT(DAY_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
-		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
+	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
+		(unsigned long)inr,(unsigned long)outr,(unsigned long)inmax,(unsigned long)outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
 	    hist->inmax = inmax; hist->outmax = outmax;
@@ -1047,8 +1070,8 @@ int absupdate;
 	plannow *= WEEK_SAMPLE;
 
 	if (plannow < now) {
-	  NEXT((now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	  NEXT((unsigned long) (now-plannow));
+	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
@@ -1060,7 +1083,7 @@ int absupdate;
 	
 	for (x=0; x<WEEK_COUNT; x++) {
 	    NEXT(WEEK_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
@@ -1074,8 +1097,8 @@ int absupdate;
 	plannow *= MONTH_SAMPLE;
 
 	if (plannow < now) {
-	  NEXT((now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	  NEXT((unsigned long) (now-plannow));
+	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
@@ -1087,7 +1110,7 @@ int absupdate;
 
 	for (x=0; x<MONTH_COUNT; x++) {
 	    NEXT(MONTH_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
@@ -1101,8 +1124,8 @@ int absupdate;
 	plannow *= YEAR_SAMPLE;
 
 	if (plannow < now) {
-	  NEXT((now-plannow));
-	  fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	  NEXT((unsigned long) (now-plannow));
+	  fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		  (unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	  hist->time = now;
 	  hist->in = inr; hist->out = outr;
@@ -1114,7 +1137,7 @@ int absupdate;
 
 	for (x=0; x<YEAR_COUNT; x++) {
 	    NEXT(YEAR_SAMPLE);
-	    fprintf(fo,"%lu %lu %lu %lu %lu\n",now,
+	    fprintf(fo,"%lu %lu %lu %lu %lu\n",(unsigned long)now,
 		(unsigned long)inr,(unsigned long)outr,inmax,outmax);
 	    hist->time = now;
 	    hist->in = inr; hist->out = outr;
@@ -1153,7 +1176,7 @@ int absupdate;
     free(lhist);
 }
 
-void init_colour(int *colmap, int c0, int c1, int c2) {
+static void init_colour(int *colmap, int c0, int c1, int c2) {
   *colmap++ = c0;
   *colmap++ = c1;
   *colmap = c2;
@@ -1272,7 +1295,7 @@ char **argv;
 	               used = 1;
 		       break;
 	           case '0':  /* assume zeroes */
-	               options |= OPTION_UNKNASZERO;
+	               options |= OPTION_WITHZEROES;
 	               used = 1;
 	               break;	
                    default:
@@ -1407,7 +1430,7 @@ char **argv;
                   for (i=1; (i<argc) && loop; i++)   /* check all args delimited by '[' and ']' */
                   { qstr = argv[argi+i];
                     /* fprintf(stderr, "qstr = \"%s\"\n", qstr); */
-                    for (j=0; (j<strlen(qstr)) && loop; j++)   /* check arg */
+                    for (j=0; ((size_t) j<strlen(qstr)) && loop; j++)   /* check arg */
                     { if (qstr[j] == '[')   /* beginning of YLegend-string */
                       { j++;        /* skip '[' */
                         *longup = 0;   /* clear */
@@ -1427,7 +1450,7 @@ char **argv;
                               (qstr[j] == ']'))
                           { /* escape okay -> append qstr[j] */
                             k = strlen(longup);
-                            if ((k + 1) > (size_t)99)
+                            if ((size_t) (k + 1) > 99)
                             { fprintf(stderr, "1: Rateup ERROR: YLegend to long ... !\n");
                               return(1);
                             }
@@ -1441,7 +1464,7 @@ char **argv;
                         }
                         else   /* append qstr[j] */
                         { k = strlen(longup);
-                          if ((k + 1) > (size_t)99)
+                          if ((size_t) (k + 1) > 99)
                           { fprintf(stderr, "3: Rateup ERROR: YLegend to long ... !\n");
                             return(1);
                           }
@@ -1463,7 +1486,7 @@ char **argv;
                       }
                     }   /* for (j=0; (j<strlen(qstr)) && loop; j++) */
                     k = strlen(longup);
-                    if ((k + 1) > (size_t)99)
+                    if ((size_t) (k + 1) > 99)
                     { fprintf(stderr, "5: Rateup ERROR: YLegend to long ... !\n");
                       return(1);
                     }
@@ -1474,7 +1497,7 @@ char **argv;
                   used = i;
                 }
                 /* remove the last space in longup: */
-                longup[max(0, strlen(longup)-1)] = 0;
+                longup[max(0, (signed) strlen(longup)-1)] = 0;
                 shortup = longup;
                 /* fprintf(stderr, "YLegend = \"%s\"\n", longup); */
                 break;
